/*
 *  S-News version 0.1.9 - A Simple News Server
 *  Copyright (C) 1998 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/stat.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#include <stdio.h>
#include <ctype.h>
#if defined(USER)
#include <pwd.h>
#elif defined(GROUP)
#include <grp.h>
#endif

#include "snews.h"

struct config
{
	struct config *next;
	char *value;
};

static void set_ug_id(void);
static struct config *read_config(char *,char *);
static int parse_active(char *,struct config *);
static int match_pat(char *,char *);

static char *progname;

int main(int argc,char **argv)
{
	struct config *cfg;

	set_ug_id();

	progname = argv[0];

	if (argc!=2)
	{
		fprintf(stderr,"Usage: %s newsfeed\n",progname);
		return (1);
	}

	if (!(cfg = read_config(CONFDIR"/newsfeeds",argv[1])))
		return (1);

	return (parse_active(CONFDIR"/active",cfg));
}

static void set_ug_id(void)
{
#if defined(USER)
	struct passwd *pwd;

	if ((pwd = getpwnam(USER)))
	{
		setregid(pwd->pw_gid,pwd->pw_gid);
		setreuid(pwd->pw_uid,pwd->pw_uid);
	}
#elif defined(GROUP)
	struct group *grp;

	if ((grp = getgrnam(GROUP)))
	{
		setregid(grp->gr_gid,grp->gr_gid);
	}
#endif
#if defined(UMASK)
	umask(UMASK);
#endif
}

static struct config *read_config(char *file,char *feed)
{
	char line[BUFLEN];
	struct config *ret = 0, *cfg, **ptr = &ret;
	FILE *fp;
	char *cp;

	if (!(fp = fopen(file,"r")))
	{
		perror(file);
		return (0);
	}
	while (fgets(line,BUFLEN,fp))
	{
		if (!(cp = strchr(line,'\n')))
		{
			fprintf(stderr,"%s: long line\n",file);
			return (0);
		}
		*cp = '\0';
		if ((cp = strchr(line,'#')))
			*cp = '\0';
		if (!*line) continue;
		if (!(cp = strchr(line,'\t')))
		{
			fprintf(stderr,"%s: malformed line\n",file);
			return (0);
		}
		*(cp++) = '\0';

		if (strcmp(line,feed) == 0)
		{
			if (!((cfg = malloc(sizeof(struct config))) &&
			      (cfg->value = strdup(cp))))
			{
				perror(progname);
				return (0);
			}
			*ptr = cfg; ptr = &cfg->next;
			cfg->next = 0;
		}
	}
	fclose(fp);

	return (ret);
}

static int parse_active(char *file,struct config *cfg)
{
	char line[BUFLEN];
	FILE *fp;
	int ret=0;

	if (!(fp = fopen(file,"r")))
	{
		perror(file);
		return (1);
	}

	while (fgets(line,BUFLEN,fp))
	{
		char *cp;
		struct config *ptr;

		if (!strchr(line,'\n'))
		{
			fprintf(stderr,"%s: long line",file);
			ret = 1;
			break;
		}

		if ((cp = strchr(line,' ')) <= line)
		{
			fprintf(stderr,"%s: malformed line\n",file);
			ret = 1;
			break;
		}

		*(cp++) = '\0';

		for (ptr=cfg; ptr; ptr=ptr->next)
		{
			if (match_pat(ptr->value,line))
				printf("%s %s",line,cp);
		}
	}

	fclose(fp);

	return (ret);
}

static int match_pat(char *pat,char *str)
{
	unsigned char c,u;

	while ((c = *(pat++)))
	{
		switch (c)
		{
		case '*':
			while (*str)
			{
				if (match_pat(pat,str))
					return (1);
				++str;
			}
			break;
		case '?':
			if (!*(str++))
				return (0);
			break;
		case '[':
			if ((u=*str))
			{
				unsigned char rev,mat=0,l=0,h;

				if ((rev = (*pat=='^'?0:1)))
					pat++;
				if ((c = *pat)) do
				{
					pat++;
					if (c=='-' && l && (h=*pat) && h!=']')
					{
						pat++;
						if (u>=l && u<=h) mat=1;
						l = 0;
					}
					else
					{
						if (u==c) mat=1;
						l = c;
					}
				}
				while ((c=*pat) && c!=']');

				if (mat==rev)
					return (0);
			}
			break;
		default:
			if (*(str++) != c)
				return (0);
			break;
		}
	}

	return (!*str);
}
