/*
 *  S-News version 0.1.9 - A Simple News Server
 *  Copyright (C) 1998 Christopher John Purnell
 *                     cjp@lost.org.uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/stat.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#ifdef STDC_HEADERS
#include <string.h>
#endif
#include <stdio.h>
#if defined(USER)
#include <pwd.h>
#elif defined(GROUP)
#include <grp.h>
#endif

#include "snews.h"

struct config
{
	struct config *next;
	char *name;
	char *value;
};

static void set_ug_id(void);
static struct config *read_config(char *);
static int match_group(char *,char *,char *);

static char *progname;

int main(int argc,char **argv)
{
	char file[BUFLEN];
	int i;
	struct config *cfg;

	set_ug_id();

	progname = argv[0];

	if (!(cfg = read_config(CONFDIR"/newsfeeds")))
		return (1);

	for (i=1; i<argc; ++i)
	{
		char *line,*end;
		struct config *ptr;

		end = strrchr(line=argv[i],'/');

		for (ptr=cfg; ptr; ptr=ptr->next)
		{
			if (match_group(ptr->value,line,end))
			{
				FILE *fp;

				sprintf(file,CONFDIR"/suck/%s/outgoing",
					ptr->name);

				if (!(fp = fopen(file,"a")))
				{
					perror(file);
					return (1);
				}

				if (fputs(line,fp) < 0 ||
				    fputc('\n',fp) < 0)
				{
					perror(file);
					fclose(fp);
					return (1);
				}

				if (fclose(fp))
				{
					perror(file);
					return (1);
				}

				break;
			}
		}
	}

	return (0);
}

static void set_ug_id(void)
{
#if defined(USER)
	struct passwd *pwd;

	if ((pwd = getpwnam(USER)))
	{
		setregid(pwd->pw_gid,pwd->pw_gid);
		setreuid(pwd->pw_uid,pwd->pw_uid);
	}
#elif defined(GROUP)
	struct group *grp;

	if ((grp = getgrnam(GROUP)))
	{
		setregid(grp->gr_gid,grp->gr_gid);
	}
#endif
#if defined(UMASK)
	umask(UMASK);
#endif
}

static struct config *read_config(char *file)
{
	char line[BUFLEN];
	struct config *ret = 0, *cfg, **ptr = &ret;
	FILE *fp;
	char *cp;

	if (!(fp = fopen(file,"r")))
	{
		perror(file);
		return (0);
	}
	while (fgets(line,BUFLEN,fp))
	{
		if (!(cp = strchr(line,'\n')))
		{
			fprintf(stderr,"%s: long line\n",file);
			return (0);
		}
		*cp = '\0';
		if ((cp = strchr(line,'#')))
			*cp = '\0';
		if (!*line) continue;
		if (!(cp = strchr(line,'\t')))
		{
			fprintf(stderr,"%s: malformed line\n",file);
			return (0);
		}
		*(cp++) = '\0';

		if (!((cfg = malloc(sizeof(struct config))) &&
		      (cfg->name = strdup(line)) &&
		      (cfg->value = strdup(cp))))
		{
			perror(progname);
			return (0);
		}
		*ptr = cfg; ptr = &cfg->next;
		cfg->next = 0;
	}
	fclose(fp);

	return (ret);
}

static int match_group(char *pat,char *str,char *end)
{
	char c;

	while ((c = *(pat++)))
	{
		switch (c)
		{
		case '*':
			while (str<end)
			{
				if (match_group(pat,str,end))
					return (1);
				++str;
			}
			break;
		case '?':
			if (str>=end)
				return (0);
			break;
		case '.':
			if (str>=end || *str!='/')
				return (0);
			break;
		case '[':
			if (str>=end)
			{
				unsigned char rev,mat=0,l=0,h,u=*str;

				if ((rev = (*pat=='^'?0:1)))
					pat++;
				if ((c=*pat)) do
				{
					pat++;
					if (c=='-' && l && (h=*pat) && h!=']')
					{
						pat++;
						if (u>=l && u<=h) mat=1;
						l = 0;
					}
					else
					{
						if (u==c) mat=1;
						l = c;
					}
				}
				while ((c=*pat) && c!=']');
				if (mat==rev)
					return (0);
			}
			break;
		default:
			if (str>=end || *str!=c)
				return (0);
			break;
		}
		++str;
	}

	return (str<=end);
}
